@@ -1,19 +1,18 @@
 package App::fatten;
 
 our $DATE = '2015-04-15'; # DATE
-our $VERSION = '0.36'; # VERSION
+our $VERSION = '0.37'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 use experimental 'smartmatch';
 use Log::Any '$log';
-use Log::Any::For::Builtins qw(system my_qx);
 BEGIN { no warnings; $main::Log_Level = 'info' }
 
 use App::tracepm (); # we need list of trace methods too so we load early
 use File::chdir;
-use File::Slurper qw(write_text read_text);
+use File::Slurper qw(write_binary read_binary);
 use version;
 
 my @ALLOW_XS = qw(List::MoreUtils version::vxs);
@@ -103,12 +102,31 @@ sub _build_lib {
     my %mod_paths; # modules to add, key=name, val=path
 
     my $deps = $self->{deps};
-    for (@$deps) {
+    for (@{$deps // []}) {
         next if $_->{is_core} && $self->{exclude_core};
         $log->debugf("  Adding module: %s (traced)", $_->{module});
         $mod_paths{$_->{module}} = undef;
     }
 
+    if ($self->{include_prereq} && @{ $self->{include_prereq} }) {
+        $log->infof("Searching recursive prereqs to add into fatpacking: %s", $self->{include_prereq});
+        for my $prereq (@{ $self->{include_prereq} }) {
+            my @mods = ($prereq);
+            # find prereq's dependencies
+            my $res = $self->_run_lcpan("deps", "-R", "--perl-version", $self->{perl_version}->numify, $prereq);
+            for my $entry (@{ $res }) {
+                $entry->{module} =~ s/^\s+//;
+                push @mods, $entry->{module};
+            }
+            # pull all the other modules from the same dists
+            $res = $self->_run_lcpan("mods-from-same-dist", "--detail", @mods);
+            for my $entry (@{ $res }) {
+                $log->debugf("  Adding module: %s (include_prereq %s, dist %s)", $entry->{name}, $prereq, $entry->{dist});
+                $mod_paths{$entry->{name}} = undef;
+            }
+        }
+    }
+
     for (@{ $self->{include} // [] }) {
         $log->debugf("  Adding module: %s (included)", $_);
         $mod_paths{$_} = undef;
@@ -167,22 +185,44 @@ sub _build_lib {
     # filter excluded
     my $excluded_distmods;
     my $excluded_list;
+    my $excluded_prereqs;
     my %fmod_paths; # filtered mods
-  MOD:
+
+  MOD_TO_FILTER:
     for my $mod (sort keys %mod_paths) {
-        if ($self->{_exclude_deps} && $self->{_exclude_deps}{$mod}) {
-            $log->infof("Excluding %s: skipped by allow_dep %s", $mod, $self->{_exclude_deps}{$mod});
-            next MOD;
+        if ($self->{exclude_prereq} && @{ $self->{exclude_prereq} }) {
+            if (!$excluded_prereqs) {
+                $excluded_prereqs = {};
+                $log->infof("Searching recursive prereqs to exclude from fatpacking: %s", $self->{exclude_prereq});
+                for my $prereq (@{ $self->{exclude_prereq} }) {
+                    my @mods = ($prereq);
+                    # find prereq's dependencies
+                    my $res = $self->_run_lcpan("deps", "-R", "--perl-version", $self->{perl_version}->numify, $prereq);
+                    for my $entry (@{ $res }) {
+                        $entry->{module} =~ s/^\s+//;
+                        push @mods, $entry->{module};
+                    }
+                    # pull all the other modules from the same dists
+                    $res = $self->_run_lcpan("mods-from-same-dist", "--detail", @mods);
+                    for my $entry (@{ $res }) {
+                        $excluded_prereqs->{$entry->{name}} = $prereq;
+                    }
+                }
+            }
+            if ($excluded_prereqs->{$mod}) {
+                $log->infof("Excluding %s: skipped by exclude_prereq %s", $mod, $excluded_prereqs->{$mod});
+                next MOD_TO_FILTER;
+            }
         }
 
         if ($self->{exclude} && $mod ~~ @{ $self->{exclude} }) {
             $log->infof("Excluding %s: skipped", $mod);
-            next MOD;
+            next MOD_TO_FILTER;
         }
         for (@{ $self->{exclude_pattern} // [] }) {
             if ($mod ~~ /$_/) {
                 $log->infof("Excluding %s: skipped by pattern %s", $mod, $_);
-                next MOD;
+                next MOD_TO_FILTER;
             }
         }
         if ($self->{exclude_dist}) {
@@ -194,7 +234,7 @@ sub _build_lib {
             }
             if ($mod ~~ @$excluded_distmods) {
                 $log->infof("Excluding %s (by dist): skipped", $mod);
-                next MOD;
+                next MOD_TO_FILTER;
             }
         }
         if (defined(my $file = $self->{exclude_list})) {
@@ -227,6 +267,8 @@ sub _build_lib {
     %mod_paths = %fmod_paths;
 
     require Module::XSOrPP;
+
+  MOD_TO_ADD:
     for my $mod (sort keys %mod_paths) {
         my $mpath = $mod_paths{$mod};
 
@@ -240,7 +282,14 @@ sub _build_lib {
         }
 
         $mpath //= Module::Path::More::module_path(module=>$mod);
-        defined $mpath or die "Can't find path for $mod\n";
+        unless (defined $mpath) {
+            if ($self->{skip_not_found}) {
+                $log->infof("Path for module '%s' not found, skipped", $mod);
+                next MOD_TO_ADD;
+            } else {
+                die "Can't find path for $mod\n";
+            }
+        }
 
         my $modp = $mod; $modp =~ s!::!/!g; $modp .= ".pm";
         my ($dir) = $modp =~ m!(.+)/(.+)!;
@@ -263,16 +312,16 @@ sub _build_lib {
                 );
             };
             $log->debug("  Stripping $mpath --> $modp ...");
-            my $src = read_text($mpath);
+            my $src = read_binary($mpath);
             my $stripped = $stripper->strip($src);
-            write_text("$tempdir/lib/$modp", $stripped);
+            write_binary("$tempdir/lib/$modp", $stripped);
         } elsif ($self->{strip}) {
             require Perl::Strip;
             my $strip = Perl::Strip->new;
             $log->debug("  Stripping $mpath --> $modp ...");
-            my $src = read_text($mpath);
+            my $src = read_binary($mpath);
             my $stripped = $strip->strip($src);
-            write_text("$tempdir/lib/$modp", $stripped);
+            write_binary("$tempdir/lib/$modp", $stripped);
         } elsif ($self->{squish}) {
             $log->debug("  Squishing $mpath --> $modp ...");
             require Perl::Squish;
@@ -290,32 +339,31 @@ sub _build_lib {
 }
 
 sub _pack {
-    require Proc::ChildError;
+    require IPC::System::Options;
 
     my $self = shift;
 
     my $tempdir = $self->{tempdir};
 
     local $CWD = $tempdir;
-    system join(
-        "",
-        "fatpack file ",
-        __sq($self->{abs_input_file}), " > ",
-        __sq($self->{abs_output_file}),
+    IPC::System::Options::system(
+        {log=>1, die=>1, shell=>1},
+        "fatpack", "file",
+        $self->{abs_input_file}, ">",
+        $self->{abs_output_file},
     );
-    die "Can't fatpack file: ".Proc::ChildError::explain_child_error()."\n" if $?;
 
     chmod 0755, $self->{abs_output_file};
 
     # replace shebang line (which contains perl path used by fatpack) with a
     # default system perl. perhaps make this configurable in the future.
     {
-        my $ct = read_text($self->{abs_output_file});
+        my $ct = read_binary($self->{abs_output_file});
         my $shebang = $self->{shebang} // '#!/usr/bin/perl';
         $shebang = "#!$shebang" unless $shebang =~ /^#!/;
         $shebang =~ s/\R+//g;
         $ct =~ s{\A#!(.+)}{$shebang};
-        write_text($self->{abs_output_file}, $ct);
+        write_binary($self->{abs_output_file}, $ct);
     }
 
     $log->infof("  Produced %s (%.1f KB)",
@@ -549,22 +597,53 @@ _
             summary => 'Pass more options to `App::tracepm`',
             tags => ['category:module-selection'],
         },
-        allow_deps => {
+        include_prereq => {
+            'summary.alt.plurality.singular' => 'Include module and its recursive dependencies for fatpacking',
+            schema => ['array*', of=>'str*'],
+            description => <<'_',
+
+This option can be used to include a module, as well as other modules in the
+same distribution as that module, as well as the distribution's recursive
+dependencies, for fatpacking. Dependencies will be searched using a local CPAN
+index. This is a convenient alternative to tracing a module. So you might want
+to use this option together with setting `trace_method` to `none`.
+
+This option requires that `lcpan` is installed and a fairly recent lcpan index
+is available.
+
+_
+            tags => ['category:module-selection'],
+        },
+        exclude_prereq => {
             'summary.alt.plurality.singular' => 'Allow script to depend on a module instead of fatpacking it',
             schema => ['array*', of=>'str*'],
-            'x.name.is_plural' => 1,
             description => <<'_',
 
 This option can be used to express that script will depend on a specified
-module. The distribution of that module and the recursive dependencies for _that
-distribution_ will be retrieved recursively using `lcpan`. All these will be
-excluded from being fatpacked.
+module, instead of including it fatpacked. The prereq-ed module, as well as
+other modules in the same distribution, as well as its prereqs and so on
+recursively, will be excluded from fatpacking as well.
 
 This option can be used to express dependency to an XS module, since XS modules
 cannot be fatpacked.
 
-This option requires that `lcpan` is installed and a fairly recent lcpan index
-is available.
+To query dependencies, a local CPAN index is used for querying speed. Thus, this
+option requires that `lcpan` is installed and a fairly recent lcpan index is
+available.
+
+_
+            tags => ['category:module-selection'],
+        },
+        skip_not_found => {
+            summary => 'Instead of dying, skip when module to add is not found',
+            'summary.alt.bool.not' => 'Instead of skipping, die when module to add is not found',
+            schema => ['bool'],
+            default => 1,
+            description => <<'_',
+
+This option is useful when you use `include_prereq`, because modules without its
+own .pm files will also be included (CPAN indexes packages, including those that
+do not have their own .pm files).
 
 _
             tags => ['category:module-selection'],
@@ -743,24 +822,6 @@ sub fatten {
     my %args = @_;
     my $self = __PACKAGE__->new(%args);
 
-    if ($self->{allow_deps} && @{ $self->{allow_deps} }) {
-        my %exclude_deps;
-        for my $dep (@{ $self->{allow_deps} }) {
-            my @dep_mods = ($dep);
-            my $res = $self->_run_lcpan("deps", "-R", $dep);
-            for my $ent (@{ $res }) {
-                $ent->{module} =~ s/^\s+//;
-                push @dep_mods, $ent->{module};
-            }
-            $res = $self->_run_lcpan("mods-from-same-dist", @dep_mods);
-            for my $ent (@{ $res }) {
-                $log->debugf("  Adding to exclude_deps: %s (allow_dep %s)", $ent, $dep);
-                $exclude_deps{$ent} = $dep;
-            }
-        }
-        $self->{_exclude_deps} = \%exclude_deps;
-    }
-
     my $tempdir = File::Temp::tempdir(CLEANUP => 0);
     $log->debugf("Created tempdir %s", $tempdir);
     $self->{tempdir} = $tempdir;
@@ -839,8 +900,10 @@ sub fatten {
     $self->{abs_output_file} //= Cwd::abs_path($output_file) or return
         [500, "Can't find absolute path of output file '$self->{output_file}'"];
 
-    $log->infof("Tracing dependencies ...");
-    $self->_trace;
+    unless ($self->{trace_method} eq 'none') {
+        $log->infof("Tracing dependencies ...");
+        $self->_trace;
+    }
 
     $log->infof("Building lib/ ...");
     $self->_build_lib;
@@ -888,7 +951,7 @@ App::fatten - Pack your dependencies onto your script file
 
 =head1 VERSION
 
-This document describes version 0.36 of App::fatten (from Perl distribution App-fatten), released on 2015-04-15.
+This document describes version 0.37 of App::fatten (from Perl distribution App-fatten), released on 2015-04-15.
 
 =head1 SYNOPSIS
 
@@ -913,19 +976,6 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<allow_deps> => I<array[str]>
-
-This option can be used to express that script will depend on a specified
-module. The distribution of that module and the recursive dependencies for I<that
-distribution> will be retrieved recursively using C<lcpan>. All these will be
-excluded from being fatpacked.
-
-This option can be used to express dependency to an XS module, since XS modules
-cannot be fatpacked.
-
-This option requires that C<lcpan> is installed and a fairly recent lcpan index
-is available.
-
 =item * B<allow_xs> => I<array[str]>
 
 =item * B<args> => I<array[str]>
@@ -974,6 +1024,20 @@ Regex patterns of modules to exclude.
 
 When you don't want to include a pattern of modules, specify it here.
 
+=item * B<exclude_prereq> => I<array[str]>
+
+This option can be used to express that script will depend on a specified
+module, instead of including it fatpacked. The prereq-ed module, as well as
+other modules in the same distribution, as well as its prereqs and so on
+recursively, will be excluded from fatpacking as well.
+
+This option can be used to express dependency to an XS module, since XS modules
+cannot be fatpacked.
+
+To query dependencies, a local CPAN index is used for querying speed. Thus, this
+option requires that C<lcpan> is installed and a fairly recent lcpan index is
+available.
+
 =item * B<include> => I<array[str]>
 
 Include extra modules.
@@ -997,6 +1061,17 @@ distribution. Will determine other modules from the C<.packlist> file.
 
 Include extra modules from a list in a file.
 
+=item * B<include_prereq> => I<array[str]>
+
+This option can be used to include a module, as well as other modules in the
+same distribution as that module, as well as the distribution's recursive
+dependencies, for fatpacking. Dependencies will be searched using a local CPAN
+index. This is a convenient alternative to tracing a module. So you might want
+to use this option together with setting C<trace_method> to C<none>.
+
+This option requires that C<lcpan> is installed and a fairly recent lcpan index
+is available.
+
 =item * B<input_file> => I<str> (default: "-")
 
 Path to input file (script to be fatpacked).
@@ -1035,6 +1110,14 @@ different sets of core modules as well as different versions of the modules.
 
 Set shebang line/path.
 
+=item * B<skip_not_found> => I<bool> (default: 1)
+
+Instead of dying, skip when module to add is not found.
+
+This option is useful when you use C<include_prereq>, because modules without its
+own .pm files will also be included (CPAN indexes packages, including those that
+do not have their own .pm files).
+
 =item * B<squish> => I<bool> (default: 0)
 
 Whether to squish included modules using Perl::Squish.
